@@ -9,7 +9,7 @@ use Carp;
 use Config;
 use Math::DCT 'dct2d';
 
-our $VERSION = '0.2_2';
+our $VERSION = '0.1';
 
 =head1 NAME
 
@@ -53,9 +53,9 @@ Image::PHash allows you to calculate the (DCT-based) perceptual hash (pHash) of
 
 The constructor and general structure is based on L<Image::Hash> - keeping usage quite
 similar, but the pHash algorithm is rewritten from scratch as the L<Image::Hash>
-implementation was flawed (and slow). Apart from fixes/tweaks for L<GD>, L<Imager>, 
-L<ImageMagick> resizing, L<Image::Imlib2> support is added along with some unique features
-like reduced hashes for indexing, options to deal with image mirroring, alternative
+implementation was flawed (and slow). Apart from fixes/tweaks for GD, Imager, ImageMagick
+resizing, L<Image::Imlib2> support is added along with some unique features like
+reduced hashes for indexing, options to deal with image mirroring, alternative
 bitmap methods.
 
 A fast DCT XS module made specifically to serve this hashing module is used: L<Math::DCT>.
@@ -68,11 +68,10 @@ Depending on your setup, it should be 15x or so faster than pHash.org
   my $iph = Image::PHash->new($image_file , $library?, \%settings?);
   
 The first argument is an image filename when the Imlib2 library is used, but it
-can also by a variable with the image data for the other libraries, or even an
-image object for the supported libraries.
+can also by a variable with the image data for the other libraries.
 
 The second (optional) argument is the image library. Valid options are C<Imlib2>,
-C<GD>, C<Imager>, C<ImageMagick> and if not specified the module will try to load
+C<GD>, C<ImageMagick>, C<Imager> and if not specified the module will try to load
 them in that order. Using a different library (or even library version) will most
 likely result in different hashes being returned, so make sure you hash your entire
 image set using the same image library. Also, the module will probably not work with
@@ -97,12 +96,12 @@ if you want a different balance of speed/quality.
 
 =item * C<imager_qtype> : The C<qtype> parameter for L<Imager> which defines the
 quality of scaling performed. The type C<mixing> is used by default as it seems to
-behave well for most cases while being about twice as fast as L<Imager>'s internal
-default qtype of C<normal>, but you can still manually specify that if you prefer.
+behave for most cases while being about twice as fast as Imager's internal default
+qtype of C<normal>, but you can still manually specify that if you prefer.
 
 The constructor will C<croak> if there is an error loading an image library or an
-incorrect/missing argument. It will only C<carp> and return C<undef> if an image
-library returns after failing to load an image.
+incorrect/missing argument. It will only C<carp> and return C<undef> if an image library
+returns after failing to load an image.
 
 =back
 
@@ -285,8 +284,7 @@ Will convert a bit value string to a hex string.
 Will calculate the bit difference of two hex string hashes (their Hamming distance
 of their bit stream form). On 64 bit systems (checking C<$Config{ivsize}>) it will
 actually call C<_diff64> which can calculate the difference of up to 64bit hashes in
-a single operation (using C<%064b>). You can call C<_diff64> directly if you prefer
-in that scenario.
+a single operation (using C<%064b>).
 
 =head1 NOTES
 
@@ -306,7 +304,7 @@ be resized to 32x32 in the end, the fastest hashing performance would be if you
 32x32 thumbnails. In that case, the performance of the libraries in the same order for
 the resized imageset were: 659 h/s, 664 h/s, 1883 h/s, 2296 h/s. It is clear that
 L<Image::Imlib2> should be preferred when hashing performance is desired, as it offers
-dramatically better performance (unless you are hashing 32x32 images in which case L<GD>
+dramatically better performance (unless you are hashing 32x32 images in which case GD
 also fast). It should be noted that the resulting hashes don't have exactly the same
 behaviour/metrics, due to the different resizing algorithms used, but the differences
 seem to be very small. You are encouraged to test on your own data set.
@@ -317,7 +315,7 @@ be compatible.
 Finally, if you are curious about the performance of this module compared to the
 C++ pHash.org implementation, pHash.org could achieve 33 h/s with the test setup as
 above, making L<Image::PHash> over 16x faster with Imlib2. With pre-sized 32x32
-images, pHash.org ran at 101 h/s (~23x slower than L<Image::PHash>/Imlin2).
+images, pHash.org ran at 101 h/s.
 
 =head2 Compatibility of hashes
 
@@ -374,8 +372,6 @@ respectively are useful for lookups (and still with lots of false positives as n
 
 =cut
 
-my $_64bit;
-
 sub pHash7 {
     my ($self, %opt) = @_;
     return $self->pHash(%opt, geometry => '7x7', reduce => 1);
@@ -409,20 +405,9 @@ sub new {
         Imager      => ['Imager', \&_reduce_Imager, \&_pixels_Imager, 1],
     );
 
-    my $ref  = ref($self->{image});
-    my $file = $ref ? undef : -f $self->{image};
+    my $file = -f $self->{image};
 
-    if ($ref) {    # Passed image object
-        $self->{module} = undef;
-        foreach (_lib_order()) {
-            my $type = $_ eq 'GD' ? 'GD::Image' : $libs{$_}->[0];
-            if ($type eq $ref) {
-                $self->{module} = $_;
-                last;
-            }
-        }
-        croak("Object of unknown type $ref.") unless $self->{module};
-    } elsif ($self->{module}) {    # User specified image module
+    if ($self->{module}) {    # User specified image module
         $libs{$libs{$_}->[0]} = $libs{$_} for keys %libs;    # Allow synonyms
         my $lib = $libs{$self->{module}}->[0];
 
@@ -435,8 +420,8 @@ sub new {
 
         $self->{module} = $lib; # Normalize
     } else {
-        # Try to load Imlib2, GD, Imager, ImageMagick in that order
-        foreach my $lib (_lib_order()) {
+        # Try to load Imlib2, GD, ImageMagic or Imager in that order
+        foreach my $lib (qw/Imlib2 GD Magick Imager/) {
             if (eval "require $libs{$lib}->[0]" && ($file || $libs{$lib}->[3])) {
                 $self->{module} = $lib;
                 last;
@@ -447,28 +432,24 @@ sub new {
         ) unless $self->{module};
     }
     croak("No file at $self->{image}")
-        unless $ref || $file || (length($self->{image}) > 255 && $libs{$self->{module}}->[3]);
+        unless $file || (length($self->{image}) > 255 && $libs{$self->{module}}->[3]);
 
     my $error = '';
     if ($libs{$self->{module}}->[0] eq 'Image::Imlib2') {
-        $self->{im} = $ref ? $self->{image} : Image::Imlib2->load($self->{image});
+        $self->{im} = Image::Imlib2->load($self->{image});
     } elsif ($libs{$self->{module}}->[0] eq 'GD') {
         GD::Image->trueColor(1);
-        $self->{im} = $ref ? $self->{image} : GD::Image->new($self->{image});
+        $self->{im} = GD::Image->new($self->{image});
     } elsif ($libs{$self->{module}}->[0] eq 'Image::Magick') {
-        if ($ref) {
-            $self->{im} = $self->{image};
-        } else {
-            $self->{im} = Image::Magick->new();
-            $error =
-                ($file)
-                ? $self->{im}->Read($self->{image})
-                : $self->{im}->BlobToImage($self->{image});
-            $self->{im} = undef if $error;
-        }
+        $self->{im} = Image::Magick->new();
+        $error =
+            ($file)
+            ? $self->{im}->Read($self->{image})
+            : $self->{im}->BlobToImage($self->{image});
+        $self->{im} = undef if $error;
     } else {
         my $type = $file ? 'file' : 'data';
-        $self->{im} = $ref ? $self->{image} : Imager->new($type => $self->{image});
+        $self->{im} = Imager->new($type => $self->{image});
         $error = Imager->errstr() || '';
     }
 
@@ -513,17 +494,10 @@ sub b2h {
     return $hex;
 }
 
-sub _lib_order {qw/Imlib2 GD Imager ImageMagick/}
-
-sub _is_64bit {
-    $_64bit //= $Config{ivsize} >= 8;
-    return $_64bit;
-}
-
 # Difference in bits between two hex strings
 # About 30% slower than using %064b directly, but this is portable to 32 bits
 sub diff {
-    return _diff64(@_) if length($_[0]) <= 16 && ($_64bit || &_is_64bit);
+    return _diff64(@_) if length($_[0]) <= 16 && $Config{ivsize} >= 8;
     my $diff;
     for (my $i = 0; $i < length($_[0]); $i += 8) {
         my $d =
@@ -694,13 +668,13 @@ sub dctdump {
 }
 
 sub printbitmatrix {
-    my $self  = shift;
-    my %opt   = $self->_validate_options(@_);
+    my ($self, %opt) = @_;
+
     my @array = $self->pHash(%opt);
     my $sep   = $opt{separator} || '';
     my $fill  = $opt{filler} || ' ';
 
-    my ($xs, $ys) = split(/x/, $opt{geometry});
+    my ($xs, $ys) = split(/x/, $opt{geometry} || '8x8');
     my $str = '';
     if ($ys) {
         for (my $i = 0; $i < $xs; $i++) {
@@ -709,7 +683,7 @@ sub printbitmatrix {
             $str .= "\n";
         }        
     } else {
-        my @matrix = ("$fill$sep");
+        my @matrix = ($fill);
         OUTER:
         for (my $i = 1; $i < $self->{dct_size}; $i++) {
             for (my $j = 0; $j <= $i; $j++) {
@@ -745,9 +719,7 @@ sub _validate_options {
         method      => 1,
         mirror      => 1,
         mirrorproof => 1,
-        reduce      => 1,
-        separator   => 1,
-        filler      => 1
+        reduce      => 1
     );
 
     foreach (keys %opt) {
@@ -838,8 +810,8 @@ sub _dct_mirrorproof {
 
 sub _apply_average {
     my $array   = shift;
-    my $skip    = shift;
-    my $extra   = shift;
+    my $skip    = shift || 0;
+    my $extra   = shift || [];
     my $average = 0;
     $average += $array->[$_] foreach $skip .. $#$array;
     $average += $_           foreach @$extra;
@@ -849,7 +821,7 @@ sub _apply_average {
 
 sub _apply_log_average {
     my $array   = shift;
-    my $skip    = shift;
+    my $skip    = shift || 0;
     my $exp     = 1 + (scalar(@$array)**(1 / 6));
     my $average = 0;
     foreach ($skip .. $#$array) {
@@ -857,7 +829,7 @@ sub _apply_log_average {
         $average += $array->[$_] < 0 ? -$add : $add;
     }
     $average /= scalar(@$array) - $skip;
-    my $thres = abs($average)**$exp;
+    my $thres = $average**$exp;
     $thres *= -1 if $average < 0;
     $array->[$_] = $array->[$_] >= $thres ? '1' : '0' foreach 0 .. $#$array;
 }
@@ -874,8 +846,8 @@ sub _apply_diff {
 
 sub _apply_median {
     my $array = shift;
-    my $skip  = shift;
-    my $extra = shift;
+    my $skip  = shift || 0;
+    my $extra = shift || [];
     my @vals  = sort {$a <=> $b} @$array, @$extra;
     my $len   = scalar @vals - $skip;
     my $thresh =
@@ -887,8 +859,8 @@ sub _apply_median {
 
 =head1 ACKNOWLEDGEMENTS
 
-Initially based on L<Image::Hash>, so some code to do with loading images, pixels
-etc has been kept/adapted.
+Initially based on L<Image::Hash>, so code to do with loading images, pixels etc
+has been kept.
 
 =head1 AUTHOR
 
